// #*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#* MxExamplesCommon.cpp *#*#*#*#*#*# (C) 2014-2021 DekTec
//

#include "MxExamplesCommon.h"

#ifdef _WIN32
    // Windows specific includes
    #include <windows.h>
    #include <stddef.h>     // For (u)intptr_t
#else
    // Linux specific includes
    #include <sys/types.h>
    #include <unistd.h>
    #include <errno.h>
    #include <stdint.h>     // For (u)intptr_t
    #include <sys/time.h>
#endif


// All code resides in MxExamples namespace
namespace MxExamples
{


//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+ Utility functions +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
//=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

const char*  FrameStatusToString(DtMxFrameStatus  Status)
{
    switch (Status)
    {
    case  DT_FRMSTATUS_OK:                  return "OK";
    case  DT_FRMSTATUS_SKIPPED:             return "SKIPPED";
    case  DT_FRMSTATUS_DISABLED:            return "DISABLED";
    case  DT_FRMSTATUS_DUPLICATE:           return "DUPLICATE";
    case  DT_FRMSTATUS_LATE:                return "LATE";
    case  DT_FRMSTATUS_NO_SIGNAL:           return "NO_SIGNAL";
    case  DT_FRMSTATUS_WRONG_VIDSTD:        return "WRONG_VIDSTD";
    case  DT_FRMSTATUS_DEV_DISCONNECTED:    return "DEV_DISCONNECTED";
    case  DT_FRMSTATUS_ERROR_INTERNAL:      return "ERROR_INTERNAL";
    case  DT_FRMSTATUS_INVALID:             return "INVALID";
    }
    return "???";
}

#ifndef _WIN32
//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- ChangeTerminalMode -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
void ChangeTerminalMode(int dir)
{
    static struct termios  OldT, NewT;

    if (dir == 1)
    {
        tcgetattr(STDIN_FILENO, &OldT);
        NewT = OldT;
        NewT.c_lflag &= ~(ICANON | ECHO);
        tcsetattr(STDIN_FILENO, TCSANOW, &NewT);
    }
    else
        tcsetattr(STDIN_FILENO, TCSANOW, &OldT);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- _kbhit -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
//
int  _kbhit()
{
    struct timeval  tv;
    fd_set  rdfs;

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&rdfs);
    FD_SET(STDIN_FILENO, &rdfs);

    select(STDIN_FILENO+1, &rdfs, NULL, NULL, &tv);
    return FD_ISSET(STDIN_FILENO, &rdfs);
}

//.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.- Sleep -.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//
void  Sleep(int  ms)
{
    usleep(ms*1000);
}

void  _getch()
{
    getchar();
}
#endif // #ifndef _WIN32

} // namespace MxExamples
